@@ -26,9 +26,73 @@
 #' @slot add.options a list to expand simulation options in the future.
 #' @slot ddf.param.ests Object of class \code{"array"}; stores the
 #'  parameters associated with the detection function.
-#' @slot results A \code{"list"} of \code{"arrays"}; stores
-#'  the estimated of abundance and density as well as other measures
-#'  of interest.
+#' @slot results A \code{"list"} with elements 'individuals' (and
+#' optionally 'clusters' and 'expected.size') as well as 'Detection'.
+#'
+#' The 'individuals' and 'clusters' elements are a list of three
+#' 3-dimensional arrays. The first is a summary array containing
+#' values for 'Area' (strata area), 'CoveredArea' (the area
+#' covered in the strata by the survey), Effort' (the line length
+#' or number of points surveyed), 'n' (the number of sightings),
+#' 'n.miss.dists' (the number of missing distances - only applicable
+#' to mixed detector types and not yet implemented in dsims), 'k'
+#' (the number of transects), 'ER' (encounter rate), 'se.ER'
+#' (standard error of the encounter rate), 'cv.ER' (coefficient of
+#' variation of the encounter rate). A value is provided for each
+#' of these for each strata as well as the region as a whole and
+#' for each simulation repetition as well as storing the mean and
+#' standard deviation of these values across simulation repetitions.
+#'
+#' The second array 'N' is the abundance estimates table. It contains
+#' values for the 'Estimate' (estimated abundance based on data from
+#' iteration i), 'se' (standard error associated with the estimate),
+#' 'cv' (coefficient of variation of estimate), 'lcl' (lower 95\%
+#' confidence interval value), 'ucl' (upper 95\% confidence interval
+#' value), 'df' the degrees of freedom associated with the estimate.
+#' A value is provided for each of these for each strata as well as
+#' the region as a whole and for each simulation repetition as well
+#' as storing the mean and standard deviation of these values across
+#' simulation repetitions.
+#'
+#' The third array 'D' is the density estimates table. It contains
+#' values for the 'Estimate' (estimated density based on data from
+#' iteration i), 'se' (standard error associated with the estimate),
+#' 'cv' (coefficient of variation of estimate), 'lcl' (lower 95\%
+#' confidence interval value), 'ucl' (upper 95\% confidence interval
+#' value), 'df' the degrees of freedom associated with the estimate.
+#' A value is provided for each of these for each strata as well as
+#' the region as a whole and for each simulation repetition as well
+#' as storing the mean and standard deviation of these values across
+#' simulation repetitions.
+#'
+#'
+#' When animals occur in clusters the expected.size element of the
+#' results list contains a 3-dimensional array. It gives values
+#' for 'Expected.S' (expected cluster size), 'se.Expected.S'
+#' (the standard error of the expected cluster size),
+#' 'cv.Expected.S' (the coefficient of variation for the expected
+#' cluster size). Values are given for each analysis strata as
+#' well as a value for the survey region as a whole and across
+#' each simulation repetition as well as overall means and standard
+#' deviations across repetitions.
+#'
+#'
+#' The Detection element of the results list is a 3-dimensional
+#' array with values for 'True.Pa' (the proportion of animals in
+#' the covered region which were detected), 'Pa' (the estimated
+#' proportion of animals detected in the covered region), 'ESW'
+#' (the estimated strip width), 'f(0)' (The estimated value of
+#' the detection function pdf at distance 0), 'SelectedModel'
+#' (the index of the model which had the best fit to the dataset
+#' for the repetition), 'DeltaCriteria' (the difference in
+#' information criteria between the best and second best fitting
+#' models where two or more models were fitted and converged),
+#' 'SuccessfulModels' (the number of models which successfully
+#' converged). Currently detection functions are pooled across
+#' all strata so there is only one global value for each
+#' simulated dataset as well as a mean value and standard
+#' deviation where appropriate.
+#'
 #' @slot warnings A \code{"list"} to store warnings and error messages encountered
 #'  during runtime.
 #' @section Methods:
@@ -238,7 +302,8 @@ histogram.N.ests <- function(x, ...){
 #' @param object object of class Simulation
 #' @param description.summary logical indicating whether an
 #'  explanation of the summary should be displayed
-#' @param ... can specify if you want the maximum number of iterations to be used where at least one model converged (use.max.reps = TRUE) or only use iterations where all models converged (use.max.reps = FALSE)
+#' @param use.max.reps by default this is FALSE meaning that only simulation repetitions where all models converged for that data set are included. By setting this to TRUE any repetition where one or more models converged will be included in the summary results.
+#' @param ... no additional arguments currently implemented
 #' @rdname summary.Simulation-methods
 #' @importFrom stats na.omit qlnorm qnorm
 #' @importFrom methods slotNames slot show
@@ -247,20 +312,13 @@ histogram.N.ests <- function(x, ...){
 setMethod(
   f="summary",
   signature="Simulation",
-  definition=function(object, description.summary = TRUE, ...){
+  definition=function(object, description.summary = TRUE, use.max.reps = FALSE, ...){
     if(description.summary){
       description.summary()
     }
     #Get number of reps
     reps <- object@reps
-    #Check for additional arguments
-    args <- list(...)
-    if (!("use.max.reps" %in% names(args))){
-      #By default exclude all iterations where one or more models failed to converge
-      use.max.reps = FALSE
-    }else{
-      use.max.reps = args$use.max.reps
-    }
+
     #Get index of iterations to use
     model.count <- length(object@ds.analysis@dfmodel)
     #These will use max iters by default
@@ -275,6 +333,93 @@ setMethod(
                                      use.max.reps = use.max.reps)
     }
 
+    #Create design summary
+    design.type = character()
+    for(i in seq(along = object@design@design)){
+      if(class(object@design) %in% c("Line.Transect.Design", "Segment.Transect.Design")){
+        design.type[i] <- switch(object@design@design[i],
+                                 systematic = "Systematic parallel line design",
+                                 eszigzag = "Equal spaced zigzag line design",
+                                 eszigzagcom = "Equal spaced zigzag with complementary lines design",
+                                 random = "Random parallel line design",
+                                 segmentedgrid = "Segmented line transect grid design")
+      }else{
+        design.type[i] <- switch(object@design@design[i],
+                                 systematic = "Systematic point design",
+                                 random = "Random sampling point design")
+
+      }
+    }
+    slots <- slotNames(object@design)
+    design.parameters <- list()
+    design.parameters$design.type <- design.type
+    for(i in seq(along = slots)){
+      if(slots[i] %in% c("design.angle", "spacing", "edge.protocol", "nested.space",
+                         "line.length", "bounding.shape", "samplers", "effort.allocation",
+                         "truncation")){
+        if(!(length(slot(object@design, slots[i])) == 0)){
+          design.parameters[[slots[i]]] <- slot(object@design, slots[i])
+        }
+      }
+    }
+
+    #Create analysis summary
+    analysis.summary <- list(dfmodels = object@ds.analysis@dfmodel,
+                             key = object@ds.analysis@key,
+                             criteria = object@ds.analysis@criteria,
+                             variance.estimator = object@ds.analysis@er.var,
+                             truncation = object@ds.analysis@truncation)
+    if(length(object@ds.analysis@cutpoints) > 0){
+      analysis.summary$cutpoints <- object@ds.analysis@cutpoints
+    }
+    #Deal with any grouping of strata
+    analysis.strata <- object@ds.analysis@group.strata
+    if(nrow(analysis.strata) > 0){
+      analysis.summary$analysis.strata <- object@ds.analysis@group.strata
+    }
+    if(length(object@ds.analysis@adjustment) > 0){
+      analysis.summary$adjustment <- object@ds.analysis@adjustment
+    }
+    if(length(object@ds.analysis@control.opts) > 0){
+      analysis.summary$control <- object@ds.analysis@control.opts
+    }
+
+    #Create population summary
+    pop.covars <- object@population.description@covariates
+
+    #Model selection table
+    tab.model.selection <- table(results$Detection[,"SelectedModel",rep.index])
+
+    #Create detectabilty summary
+    detectability.summary <- list(key.function = object@detectability@key.function,
+                                  scale.param = object@detectability@scale.param,
+                                  shape.param = object@detectability@shape.param,
+                                  cov.param = object@detectability@cov.param,
+                                  truncation = object@detectability@truncation)
+
+    # If there were no valid reps
+    if(is.null(results)){
+      #Create simulation summary object
+      summary.x <- new(Class = "Simulation.Summary",
+                       region.name = object@design@region@region.name,
+                       strata.name = object@design@region@strata.name,
+                       total.reps = object@reps,
+                       failures = reps - length(rep.index),
+                       use.max.reps = use.max.reps,
+                       individuals = list(),
+                       clusters = list(),
+                       expected.size = data.frame(),
+                       population.covars = pop.covars,
+                       detection = data.frame(),
+                       model.selection = tab.model.selection,
+                       design.summary = design.parameters,
+                       detectability.summary = detectability.summary,
+                       analysis.summary = analysis.summary)
+
+      return(summary.x)
+
+    }
+
     #Create function to calculate RMSE
     calc.RMSE <- function(x, reps){
       true.x <- x[(reps+1)]
@@ -285,8 +430,6 @@ setMethod(
     #Calculate true values
     strata.names <- object@design@region@strata.name
     strata.order <- NULL
-    #Deal with any grouping of strata
-    analysis.strata <- object@ds.analysis@group.strata
     # Get density summary
     density.summary <- summary(object@population.description@density)@summary
     if(nrow(analysis.strata) > 0){
@@ -397,6 +540,7 @@ setMethod(
                                      mean.n = results$individuals$summary[,"n","mean"],
                                      mean.n.miss.dist = ifelse("n.miss.dist" %in% dimnames(results$individuals$summary)[[2]], results$individuals$summary[,"n.miss.dist","mean"], NA),
                                      no.zero.n = zero.n,
+                                     mean.k = ifelse("k" %in% dimnames(results$individuals$summary)[[2]], results$individuals$summary[,"k","mean"], NA),
                                      mean.ER = results$individuals$summary[,"ER","mean"],
                                      mean.se.ER = results$individuals$summary[,"se.ER","mean"],
                                      sd.mean.ER = results$individuals$summary[,"ER","sd"])
@@ -408,6 +552,10 @@ setMethod(
     if(all(individual.summary$no.zero.n == 0)){
       eval(parse(text = paste("individual.summary <- subset(individual.summary, select = -no.zero.n)")))
     }
+    # For backwards compatability
+    if(all(is.na(individual.summary$mean.k))){
+      eval(parse(text = paste("individual.summary <- subset(individual.summary, select = -mean.k)")))
+    }
     individual.N <- data.frame(Truth = true.N.individuals,
                                mean.Estimate = results$individuals$N[,"Estimate","mean"],
                                percent.bias = (results$individuals$N[,"Estimate","mean"] - true.N.individuals)/true.N.individuals*100,
@@ -498,66 +646,6 @@ setMethod(
     #Find how many iterations failed
     no.fails <- reps - included.reps
     individuals <- list(summary = individual.summary, N = individual.N, D = individual.D)
-    #Model selection table
-    tab.model.selection <- table(results$Detection[,"SelectedModel",rep.index])
-    #Create detectabilty summary
-    detectability.summary <- list(key.function = object@detectability@key.function,
-                                  scale.param = object@detectability@scale.param,
-                                  shape.param = object@detectability@shape.param,
-                                  cov.param = object@detectability@cov.param,
-                                  truncation = object@detectability@truncation)
-
-    #Create design summary
-    design.type = character()
-    for(i in seq(along = object@design@design)){
-      if(class(object@design) %in% c("Line.Transect.Design", "Segment.Transect.Design")){
-        design.type[i] <- switch(object@design@design[i],
-                              systematic = "Systematic parallel line design",
-                              eszigzag = "Equal spaced zigzag line design",
-                              eszigzagcom = "Equal spaced zigzag with complementary lines design",
-                              random = "Random parallel line design",
-                              segmentedgrid = "Segmented line transect grid design")
-      }else{
-        design.type[i] <- switch(object@design@design[i],
-                              systematic = "Systematic point design",
-                              random = "Random sampling point design")
-
-      }
-    }
-    slots <- slotNames(object@design)
-    design.parameters <- list()
-    design.parameters$design.type <- design.type
-    for(i in seq(along = slots)){
-      if(slots[i] %in% c("design.angle", "spacing", "edge.protocol", "nested.space",
-                         "line.length", "bounding.shape", "samplers", "effort.allocation",
-                         "truncation")){
-        if(!(length(slot(object@design, slots[i])) == 0)){
-          design.parameters[[slots[i]]] <- slot(object@design, slots[i])
-        }
-      }
-    }
-
-    #Create analysis summary
-    analysis.summary <- list(dfmodels = object@ds.analysis@dfmodel,
-                             key = object@ds.analysis@key,
-                             criteria = object@ds.analysis@criteria,
-                             variance.estimator = object@ds.analysis@er.var,
-                             truncation = object@ds.analysis@truncation)
-    if(length(object@ds.analysis@cutpoints) > 0){
-      analysis.summary$cutpoints <- object@ds.analysis@cutpoints
-    }
-    if(nrow(analysis.strata) > 0){
-      analysis.summary$analysis.strata <- object@ds.analysis@group.strata
-    }
-    if(length(object@ds.analysis@adjustment) > 0){
-      analysis.summary$adjustment <- object@ds.analysis@adjustment
-    }
-    if(length(object@ds.analysis@control.opts) > 0){
-      analysis.summary$control <- object@ds.analysis@control.opts
-    }
-
-    #Create population summary
-    pop.covars <- object@population.description@covariates
 
     #Create simulation summary object
     summary.x <- new(Class = "Simulation.Summary",
